@@ -50,6 +50,8 @@ struct kcpp_params {
     float xtc_probability      = 0;
     float   dynatemp_range     = 0.0f;  // enables DynaTemp if neq 0. dynatemp_min = temperature - dt_range, dynatemp_max = temperature + dt_range
     float   dynatemp_exponent  = 1.0f;
+    float adaptive_target     = -1.0f; // 0.0 - 1.0, <=0.0 is disabled
+    float adaptive_decay      = 0.9f;
 
     std::string model_filename       = ""; // model path
     std::string prompt               = "";
@@ -536,6 +538,7 @@ struct savestate_data
     std::vector<uint8_t> current_draft_savestate_buffer;
     std::vector<gpt_vocab::id> savestate_context_tokens; //for context clones
     int64_t last_used = 0; //unix timestamp, updated on save or load
+    std::string media_signature = "";
 };
 
 const float default_norm_eps = 1e-5f;
